#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os
import copy


import technoap.sitcp as sitcp
import technoap.device.apu101 as apu101

	
#device IP address
DEST_IPADDR = "192.168.10.128"

##########################################################
def main(argv):

	print ("start HV configuration...")

	try:
		#create 'RBCP' instance
		rbcp = sitcp.rbcp.RBCP(True)
		rbcp.setdefaultdest(DEST_IPADDR)

		########################################################
		config = apu101.configHV.ConfigHV(rbcp)
		init_config(config)
		########################################################

	except Exception as ex:
		print (ex)

	print ("HV configuration complete...")

##########################################################
def init_config(config):

	#Pass parameter(reg_list) in dictionary format.
	#Contact TechnoAP for the value as it is a device-specific value.
	reglist = {
		"PDO": 0x7fc3,	#POS: setting offset
		"PDF": 0x7ead,	#POS: setting fine gain
		"NDO": 0x7fbb,	#NEG: setting offset
		"NDF": 0x7e19,	#NEG: setting value fine gain
		"PVO": 0x803a,	#POS: Voltage monitor value offset
		"PVF": 0x80f0,	#POS: Voltage monitor value fine gain
		"NVO": 0x8038,	#NEG: Voltage monitor value offset
		"NVF": 0x74c5,	#NEG: Voltage monitor value fine gain
		"PIO": 0x8000,	#POS: Current monitor value offset
		"PIF": 0x21a1,	#POS: Current monitor fine gain
		"NIO": 0x8004,	#NEG: Current monitor offset
		"NIF": 0x210e	#NEG: Current monitor fine gain
	}
	config.setDeviceSpecificConstants(reglist)

	#*About config.toDigitHSW, config.toDigitHVD
	#The second argument specifies the maximum output of the implemented high voltage power supply module
	#This sample program assumes that a high voltage module of 4KV is implemented.
	config.mStatus["HSW",] = config.toDigitHSW(500.0, apu101.constant.HVMODULE_IMPLEMENTED_4KV)		#500 V/min
	config.mStatus["HVD",] = config.toDigitHVD(200.0, apu101.constant.HVMODULE_IMPLEMENTED_4KV)		#200V
	config.mStatus["HSP",] = apu101.constant.REG_HSP_NEG	#negative
	config.mStatus["HSJ",] = config.toDigitHSJ(-11.0)		#-11.0V
	result = config.writeStatusAll()

	if result == False:
		raise Exception('config write error...')


##########################################################
if __name__ == "__main__":
	main(sys.argv)


