#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import time

from . import common
from . import constant
from . import register
from ..common import registerRW

from ... import num

########## target register list ##########
#target register list(common)
TARGET_REGISTER_LIST_CMN = [
	#"MOD", "MMD", "AQS", "MTM", "CLR",
	"MOD", "MMD", "MTM",
	#"RQH", "CLS", "SCS", "FRN", "TGE",
	"CLS",
	#"TSO", "TPO", "TLV", "DAC", "RRG",
	"DAC", "RRG",
	"RCH" 
]

#target register list(CH)
TARGET_REGISTER_LIST_CHN = [
	"ACG", "ADG", "FFD", "FFI", "SFR",
	"SFP", "FPZ", "SPZ", "FTH", "LLD",
	"ULD", "STH", "PUR", "POL", "WVS",
	"DCG", "DFG", "TMS", "CFF", "CFD",
	"IHW", "CLT", "CDT", "DIF", "PZD",
	"FGD", "BTS", "BRS"
]

#target register list(all)
TARGET_REGISTER_LIST = TARGET_REGISTER_LIST_CMN + TARGET_REGISTER_LIST_CHN



class Config(registerRW.RegisterRW):
	
	###############################################
	def __init__(self, rbcp = None):
		self.__initializing(True)
		super(Config, self).__init__(rbcp)

		self.mRegisterAttribute = register.Register()
		self.createRegisterInfo(TARGET_REGISTER_LIST, self.mRegisterAttribute)
	
	###############################################
	def __initializing(self, runParent = False):
		if runParent == True:
			super(Config, self)._RegisterRW__initializing()
			self.clearStatus()
	
	###############################################
	def clearStatus(self):
		super(Config, self).clearStatus()
		self.mStatus[("RRG",)] = [0, 0, 0, 0, 0, 0, 0, 0]
		self.mStatus[("RCH",)] = [0, 0, 0, 0]

	###############################################
	def isCommonRegister(self, name):
		return (TARGET_REGISTER_LIST_CMN.count(name) > 0)
	
	###############################################
	def isChannelRegister(self, name):
		return (TARGET_REGISTER_LIST_CHN.count(name) > 0)

	###############################################
	def startAcquisition(self, ipaddr = None, port = None):
		reg_stat = [constant.REG_AQS_STOP, constant.REG_AQS_START]
		result = self.writeRegisterNtimes("AQS", reg_stat, 0, ipaddr, port)
		return result

	###############################################
	def stopAcquisition(self, ipaddr = None, port = None):
		result = self.writeRegisterNtimes("AQS", [constant.REG_AQS_STOP], 0, ipaddr, port)
		return result

	###############################################
	def clearAcquisitionData(self, ipaddr = None, port = None):
		reg_stat = [constant.REG_CLR_LOW, constant.REG_CLR_HIGH, constant.REG_CLR_LOW]
		result = self.writeRegisterNtimes("CLR", reg_stat, 0, ipaddr, port)
		return result

	###############################################
	def setAcquisitionMode(self, value = constant.REG_MOD_HISTOGRAM, ipaddr = None, port = None):
		result = self.writeRegisterByName("MOD", value, ipaddr, port)
		return result

	###############################################
	def setAcquisitionTimeMode(self, value = constant.REG_MMD_REALTIME, ipaddr = None, port = None):
		result = self.writeRegisterByName("MMD", value, ipaddr, port)
		return result

	###############################################
	def setAcquisitionTime(self, seconds, ipaddr = None, port = None):
		value = common.calcSeconds2Digit(seconds)
		result = self.writeRegisterByName("MTM", value, ipaddr, port)
		return result

	###############################################
	def getAcquisitionTime(self, ipaddr = None, port = None):
		value = self.readStatus("MTM", ipaddr, port)
		result = common.calcDigit2Seconds(value)
		self.mStatus["MTM",] = value
		return result

	###############################################
	@staticmethod
	def toDigitTLV(value):
		return (value + 8192)

	###############################################
	@staticmethod
	def toDigitTPO(tpo, scs):
		value0 = (tpo / constant.TIME_DIGIT2NANOSEC) * (2 ** scs)
		value = num.bitconverter.inRangeAndCoarce(value0, constant.REG_TPO_MIND, constant.REG_TPO_MAXD)
		return value

	###############################################
	@staticmethod
	def toDigitSFR(nsec):
		return common.calcNanoseconds2Digit(nsec)

	###############################################
	@staticmethod
	def toDigitSFP(sfr_nsec, sfp_nsec):
		digit0 = Config.toDigitSFR(sfr_nsec)
		digit1 = common.calcNanoseconds2Digit(sfp_nsec)
		return (digit0 + digit1)

	###############################################
	@staticmethod
	def toDigitDFG(ngain):
		ngain0 = num.bitconverter.inRangeAndCoarce(ngain, constant.REG_DFG_MIN, constant.REG_DFG_MAX)
		value = int((ngain0 * constant.REG_DFG_COEFFICIENTA) + constant.REG_DFG_COEFFICIENTB)
		return value

	###############################################
	@staticmethod
	def toDigitCFF(code):
		return (code + 1)

	###############################################
	@staticmethod
	def toDigitIHW(usec):
		nsec = common.calcNanoseconds2Digit(usec * 1000)
		return nsec

	###############################################
	@staticmethod
	def toDigitFGD(code):
		return num.bitconverter.inRangeAndCoarce(code, constant.REG_FGD_MIN, constant.REG_FGD_MAX)

