#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import socket

from . import constant
from . import rbcpraw
from ..num import byteorder, bitconverter


class RBCPBase(rbcpraw.RBCPRaw):
	
	#########################################################
	def __init__(self, initRun = False, src_port = rbcpraw.AUTOASSIGN_PORT, src_ipaddr = rbcpraw.DEFAULT_SRC_IPADDR):
		super(RBCPBase, self).__init__(initRun, src_port, src_ipaddr)

	#########################################################
	#####read################################################
	#########################################################
	def __readNbit(self, address, nbytes, destipaddr = None, destport = None):
		
		value = None

		try:
			txbytes = abs(nbytes)
			numa = self.requestReading(address, txbytes, destipaddr, destport)

			###################################################
			if txbytes == 6:	#case 48bit --> treat as 64 bit
				numa.insert(0, 0)
				numa.insert(0, 0)
			###################################################

			if self.mBigendianMachine == False:		#platform is little endian
				numa.reverse()						# --> to host byte order

			if txbytes == 1:		#byte
				value = bitconverter.toIntU8(numa)
				value = (value & 0x0ff) if nbytes > 0 else bitconverter.toSignedNumber(value, txbytes)
			elif txbytes == 2:		#short
				value = bitconverter.toIntU16(numa)
				value = (value & 0x0ffff) if nbytes > 0 else bitconverter.toSignedNumber(value, txbytes)
			elif txbytes == 4:		#int
				value = bitconverter.toIntU32(numa)
				value = (value & 0x0ffffffff) if nbytes > 0 else bitconverter.toSignedNumber(value, txbytes)
			elif txbytes == 8 or txbytes == 6:	#quad word or 48bit
				value = bitconverter.toIntU64(numa)

				if txbytes == 8:	#quad word
					value = (value & 0x0ffffffffffffffff) if nbytes > 0 else bitconverter.toSignedNumber(value, txbytes)
				else:				#48bit
					value = (value & 0x0ffffffffffff) if nbytes > 0 else bitconverter.toSignedNumber(value, txbytes)
			else:
				pass
		except Exception as ex:
			self.mLastException = ex

		return	value

	#########################################################
	def read8bit(self, address, destipaddr = None, destport = None):
		return self.__readNbit(address, 1, destipaddr, destport)

	#########################################################
	def read16bit(self, address, destipaddr = None, destport = None):
		return self.__readNbit(address, 2, destipaddr, destport)

	#########################################################
	def read32bit(self, address, destipaddr = None, destport = None):
		return self.__readNbit(address, 4, destipaddr, destport)

	#########################################################
	def read48bit(self, address, destipaddr = None, destport = None):
		return self.__readNbit(address, 6, destipaddr, destport)

	#########################################################
	def read64bit(self, address, destipaddr = None, destport = None):
		return self.__readNbit(address, 8, destipaddr, destport)

	#########################################################
	def read8bitSigned(self, address, destipaddr = None, destport = None):
		return self.__readNbit(address, -1, destipaddr, destport)

	#########################################################
	def read16bitSigned(self, address, destipaddr = None, destport = None):
		return self.__readNbit(address, -2, destipaddr, destport)

	#########################################################
	def read32bitSigned(self, address, destipaddr = None, destport = None):
		return self.__readNbit(address, -4, destipaddr, destport)

	#########################################################
	def read48bitSigned(self, address, destipaddr = None, destport = None):
		return self.__readNbit(address, -6, destipaddr, destport)

	#########################################################
	def read64bitSigned(self, address, destipaddr = None, destport = None):
		return self.__readNbit(address, -8, destipaddr, destport)

	#########################################################
	#####write###############################################
	#########################################################
	#########################################################
	def __writeNbit(self, address, nbytes, value, destipaddr = None, destport = None):
		
		result = False

		try:
			if nbytes == 1:
				numa = bitconverter.getBytesInt8(value)
			elif nbytes == 2:
				numa = bitconverter.getBytesInt16(value)
			elif nbytes == 4:
				numa = bitconverter.getBytesInt32(value)
			elif nbytes == 6:
				numa = bitconverter.getBytesInt48(value)
			elif nbytes == 8:
				numa = bitconverter.getBytesInt64(value)
			else:
				numa = None

			##################################################################
			if self.mBigendianMachine == False:		#platform is little endian
				numa.reverse()						#  --> network byte order
			##################################################################

			result = self.requestWriting(address, nbytes, numa, destipaddr, destport)

		except Exception as ex:
			self.mLastException = ex

		return result
		
	#########################################################
	def write8bit(self, address, value, destipaddr = None, destport = None):
		return	self.__writeNbit(address, 1, value, destipaddr, destport)
	
	#########################################################
	def write16bit(self, address, value, destipaddr = None, destport = None):
		return	self.__writeNbit(address, 2, value, destipaddr, destport)

	#########################################################
	def write32bit(self, address, value, destipaddr = None, destport = None):
		return	self.__writeNbit(address, 4, value, destipaddr, destport)

	#########################################################
	def write48bit(self, address, value, destipaddr = None, destport = None):
		return	self.__writeNbit(address, 6, value, destipaddr, destport)

	#########################################################
	def write64bit(self, address, value, destipaddr = None, destport = None):
		return	self.__writeNbit(address, 8, value, destipaddr, destport)


